@@ -135,6 +135,9 @@ setGeneric("tinyErr", function(x, ...) standardGeneric("tinyErr"))
 setMethod("tinyErr", signature(x = "OM"),
           function(x, obs=TRUE, imp=TRUE, proc=TRUE, grad=TRUE, silent=FALSE) {
             OM <- x
+            if (length(OM@cpars)>0) 
+              warning("Note that this function doesn't apply to parameters in cpars.\n Must be removed manually e.g `OM@cpars$Perr_y <- NULL`")
+            
             if (!inherits(OM, 'OM')) stop("Object must be class `OM`", call.=FALSE)
             OMperf <- new("OM", DLMtool::Albacore, DLMtool::Generic_Fleet,
                           DLMtool::Perfect_Info, DLMtool::Perfect_Imp)
@@ -150,7 +153,7 @@ setMethod("tinyErr", signature(x = "OM"),
             }
             if (proc) {
               if (!silent) message("Removing all Process Error")
-              vars <- c("cv", "sd", "Perr")
+              vars <- c("cv", "sd", "Perr", "AC")
               nms <- c(slotNames('Stock'), slotNames('Fleet'))
               ind <- unique(grep(paste(vars, collapse = "|"), nms, value = FALSE))
               for (X in seq_along(ind)) {
@@ -246,6 +249,8 @@ MPtype <- function(MPs=NA) {
   if (class(MPs) != 'character') stop("MPs must be characters")
   
   Data <- DLMtool::SimulatedData
+  dims <- dim(Data@Ind)
+  # Data@RInd <- array(Data@Ind, dim=c(dims[1],3,dims[2]))
   
   runMPs <- applyMP(Data, MPs, reps = 2, nsims=1, silent=TRUE)
   recs <- runMPs[[1]]
@@ -379,7 +384,8 @@ plotFun <- function(class = c("MSE", "Data"), msg = TRUE) {
 #' @param funcs A character vector of management procedures
 #' @param noCV Logical. Should the CV slots be left out? 
 #' @author T. Carruthers
-#' @return A matrix of MPs and their required data in terms of Data slotnames
+#' @return A matrix of MPs and their required data in terms of `slotnames('Data')`,
+#' and broad Data classes for each MP
 #' @examples 
 #' Required(c("DCAC", "AvC"))
 #' Required() # For all MPs
@@ -420,6 +426,98 @@ Required <- function(funcs = NA, noCV=FALSE) {
     }
   }
   
+  # Add Data Classes 
+  dfout$DataClass <- NULL
+  for (i in 1:nrow(dfout)) {
+    DataClass <- NULL
+    
+    # Catch Data 
+    if (grepl("LHYear", dfout$Data[i]) & grepl("Cat", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Recent Catches')
+    }
+    if (!grepl("LHYear", dfout$Data[i]) & grepl("Cat", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Historical Catches')
+    }
+    if (grepl("AvC", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Average Catch')
+    }
+    
+    # Index 
+    if (grepl("Ind", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Index of Abundance')
+    }
+    
+    # Rec
+    if (grepl("Rec", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Recruitment Index')
+    }
+    
+    # Rec
+    if (grepl("Rec", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Recruitment Index')
+    }
+    
+    if (grepl("Abun", dfout$Data[i]) | grepl("SpAbun", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Current Abundance')
+    }
+    
+    if (grepl("Dep", dfout$Data[i]) | grepl("Dt", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Current Depletion')
+    }
+    
+    if (grepl("Mort", dfout$Data[i]) | grepl("MaxAge", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Natural Mortality')
+    }
+    
+    
+    if (grepl("FMSY_M", dfout$Data[i]) | grepl("BMSY_B0", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Reference Ratios')
+    }
+    
+    if (grepl("L50", dfout$Data[i]) | grepl("L95", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Maturity')
+    }
+    
+    if (grepl("ML", dfout$Data[i]) | grepl("Lbar", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Mean Length')
+    }
+    
+    if (grepl("Lc", dfout$Data[i]) | grepl("LFC", dfout$Data[i]) | grepl("LFS", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Selectivity')
+    }
+    
+    if (grepl("CAA", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Age Composition')
+    }
+    
+    if (grepl("vbK", dfout$Data[i]) | grepl("vbLinf", dfout$Data[i]) | 
+        grepl("vbt0", dfout$Data[i]) | grepl("LenCV", dfout$Data[i]) | 
+        grepl("wla", dfout$Data[i]) | grepl("wlb", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Growth Parameters')
+    }
+    
+    if (grepl("steep", dfout$Data[i]) | grepl("sigmaR", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Stock-Recruitment')
+    } 
+    
+    if (grepl("CAL_bins", dfout$Data[i]) | grepl("CAL", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Length Composition')
+    }    
+    
+    if (grepl("Cref", dfout$Data[i]) | grepl("Iref", dfout$Data[i]) | 
+        grepl("Bref", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Reference Levels')
+    } 
+    
+    if (grepl("MPrec", dfout$Data[i]) | grepl("MPeff", dfout$Data[i])) {
+      DataClass <- c(DataClass, 'Recent Management')
+    }
+    
+    DataClass <- sort(DataClass)
+    dfout$DataClass[i] <- paste0(DataClass, collapse=", ")
+    
+  }
+  
   as.matrix(dfout)
 }
 
