@@ -10,7 +10,7 @@ for (dat in dats) {
     Data2csv(get(dat), file, simno = sim,overwrite=T)
     readDat <- new("Data", file)
     for (sl in slotNames('Data')) {
-      if (!sl %in% c("TAC", "Sense", "MPrec")) {
+      if (!sl %in% c("TAC", "Sense", "MPrec", "PosMPs", "MPs")) {
         orig <- slot(get(dat), sl)
         if (class(orig) == "integer") orig <- as.numeric(orig)
         read <- slot(readDat, sl)
@@ -20,7 +20,11 @@ for (dat in dats) {
           if (length(och)<1) och <- 0
           testthat::expect_equal(och, nchar(read))
         } else {
-          if (class(orig)=="matrix") testthat::expect_equal(orig[sim,], read[sim,])
+          if (class(orig)=="matrix") {
+            nonna <- which(!is.na(orig[sim,]))
+            if (length(nonna)>0)
+              testthat::expect_equal(orig[sim,nonna], read[sim,nonna])
+          }
           if (class(orig)=="numeric") testthat::expect_equal(orig[sim], read[sim])
         }
       }
